/*!
\mainpage
CMaterialBank.cpp
Menu Disc Demo

Author: Jeroen Verbeek
Copyright (c) Jeroen Verbeek, 2003-2004. All rights reserved.
                               
This software is provided without any guarantee.
No responsibility taken for any problems caused by the use if this software.

Anyone is authorised to use this program code and methods without warranty,
all that is required is that you contact me (jeroen@happymakinggames.com),
and tell me that you use it and what you use it in -
all I want is to know is who uses it and where.

I will not ask for payment in any way: this code is free.
If you encounter a bug with the code, please tell me about it.
If you have a bug-fix for the code please provide me with it.
If you think of an improvement please tell me about it.
You are not required to provide me with bug reports, bug fixes or improvements.
If you require me to perform any work related to this code or
not related to this code, please contact me: I am always open to offers.

Pie menus are in no way my original idea, I stumbled on the idea a few years ago,
specifically Don Hopkins' http://catalog.com/hopkins/piemenus/piemenus-hopkins.html.
I am a big fan of circular menus (disc menus).
I am of the opinion that they provide the quickest method of selecting an action
from a structured tree of actions. A menu of actions organised following this method
will, in time, allow almost "blind selection" of actions. Each action selection
requires the same distance of mouse movement, just in a slightly diferent angle.
In my opinion this seems to work best with menu discs of no more than 8 items.
There is plenty of research to be found through the internet regarding the 
advantages of circular menus, for example: muscle memory.
Some websites I recommend you look at are:

http://catalog.com/hopkins/piemenus/piemenus-hopkins.html

http://www.piemenus.com/

http://www.sm.luth.se/~david/carl/www/argonaut.html

Contact: Jeroen Verbeek, jeroen@happymakinggames.com, Sydney NSW Australia
*/
/*-----------------------------------------------------------------------------------------------*/
#include <opengl/gl.h>
#include <opengl/glu.h>
#include <glut/glut.h>
#include <vector>
#include "CMaterialBank.h"

/*-----------------------------------------------------------------------------------------------*/
/*!
	Global material bank object. Needs to be initialised before use.
*/
/*-----------------------------------------------------------------------------------------------*/
CMaterialBank *g_pMaterialBank;


/*-----------------------------------------------------------------------------------------------*/
/*!
Static member array containing default materials.
*/
/*-----------------------------------------------------------------------------------------------*/
const GL_MATERIAL CMaterialBank::m_DefaultMaterials[] = {
	/*! DEFAULT MATERIAL */
	{	1.00f, 1.00f, 1.00f, 0.00f,
		RGBA_BLACK,
		RGBA_BLACK,
		RGBA_BLACK, 0.0f },

	/*! MENU_UNAVAILABLE Normal unselected and focused. */
	{	1.00f, 0.00f, 0.00f, 0.75f,
		RGBA_BLACK,
		RGBA_BLACK,
		RGBA_BLACK, 0.0f },
	
	/*! MENU_NORMAL Normal unselected and focused. */
	{	0.40f, 1.00f, 1.00f, 0.75f,
		RGBA_BLACK,
		RGBA_BLACK,
		RGBA_BLACK, 0.0f },

	/*! MENU_OUTSIDE Higlighted and focused (this is when the mouse is outside the circle) */
	{	0.00f, 1.00f, 0.00f, 0.75f,	
		RGBA_BLACK,
		RGBA_BLACK,
		RGBA_BLACK, 0.0f },

	/*! MENU_INSIDE Selected and focused (this is when the mouse is in the circle) */
	{	1.00f, 1.00f, 1.00f, 0.90f,
		RGBA_BLACK,
		RGBA_BLACK,
		RGBA_BLACK, 0.0f }
};



/*-----------------------------------------------------------------------------------------------*/
/*!
Constructor, inititalise custom materials vector.
*/
/*-----------------------------------------------------------------------------------------------*/
CMaterialBank::CMaterialBank(void)
{
	m_CustomMaterials.reserve(CUSTOM_MATERIAL_BLOCK_SIZE);
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Destructor, remove custom materials vector.
 */
/*-----------------------------------------------------------------------------------------------*/
CMaterialBank::~CMaterialBank(void)
{
	std::vector<GL_MATERIAL*>::iterator i;
	
	for(i = m_CustomMaterials.begin(); i != m_CustomMaterials.end(); i++)
		m_CustomMaterials.erase(i);
}



/*-----------------------------------------------------------------------------------------------*/
/*!
Sets the passed default material number. Performs some range checking
before setting material, if material is not in range then the default (white) 
material is set.

\param		iMaterial	The number of the default material to set.

\return		True if successful, false if not.
 */
/*-----------------------------------------------------------------------------------------------*/
bool CMaterialBank::SetMaterial(int iMaterial)
{
	if((iMaterial < 0) || (iMaterial > MAX_MATERIALS)){
		GLSetMaterial(&m_DefaultMaterials[0]);
		return false;
	}

	GLSetMaterial(&m_DefaultMaterials[iMaterial]);
	return true;
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Sets the passed material with OpenGL. Can also be used to set a material that is not
"registered" by the CMaterialBank object.

\param		pMaterial	The GL_MATERIAL to set.
 */
/*-----------------------------------------------------------------------------------------------*/
void CMaterialBank::GLSetMaterial(const GL_MATERIAL* pMaterial)
{
	glMaterialfv(GL_FRONT, GL_AMBIENT, (GLfloat*)&pMaterial->Ambient);
	glMaterialfv(GL_FRONT, GL_SPECULAR, (GLfloat*)&pMaterial->Specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, (GLfloat*)&pMaterial->Shininess);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, (GLfloat*)&pMaterial->Diffuse);
	
}


/*-----------------------------------------------------------------------------------------------*/
/*!
Sets the passed custom material number. Performs some range checking
before setting material, if material is not in range then the default (white) 
material is set.

\param		iMaterial	The number of the custom material to set.

\return		True if successful, false if not.
 */
/*-----------------------------------------------------------------------------------------------*/
bool CMaterialBank::SetCustomMaterial(int iMaterial)
{
	if((iMaterial>=0) && (iMaterial<(int)m_CustomMaterials.size())){

		GLSetMaterial(m_CustomMaterials[iMaterial]);
		return true;
	}

	GLSetMaterial(&m_DefaultMaterials[0]);
	return false;
}



/*-----------------------------------------------------------------------------------------------*/
/*!
Get the default material at the passed index and put it at pMaterial.

\param		iMaterial	The material number to get.
\param		pMaterial	A GL_MATERIAL to point at our material.

\return		True if successful, false if not.
 */
/*-----------------------------------------------------------------------------------------------*/
bool CMaterialBank::GetMaterial(int iMaterial, GL_MATERIAL* pMaterial)
{
	if((iMaterial<0) || (iMaterial>MAX_MATERIALS)){
		return false;
	}
	*pMaterial = m_DefaultMaterials[iMaterial];
	return true;
}



/*-----------------------------------------------------------------------------------------------*/
/*!
Get the custom material at the passed index and put it at pMaterial.

\param		iMaterial	The material number to get.
\param		pMaterial	A GL_MATERIAL to point at our material.

\return		True if successful, false if not.
 */
/*-----------------------------------------------------------------------------------------------*/
bool CMaterialBank::GetCustomMaterial(int iMaterial, GL_MATERIAL* pMaterial)
{
	if((iMaterial<0) || (iMaterial>(int)m_CustomMaterials.size()))
		return false;

	pMaterial = m_CustomMaterials[iMaterial];
	return true;
}



/*-----------------------------------------------------------------------------------------------*/
/*!
Add a custom material to our custom material array.

\param		pMaterial	A GL_MATERIAL to put into our custom material array.
 */
/*-----------------------------------------------------------------------------------------------*/
void CMaterialBank::AddCustomMaterial(GL_MATERIAL* pMaterial)
{
	m_CustomMaterials.push_back(pMaterial);
}



