/*!
\mainpage
CMaterialBank.h
Menu Disc Demo

Author: Jeroen Verbeek
Copyright (c) Jeroen Verbeek, 2003-2004. All rights reserved.
                               
This software is provided without any guarantee.
No responsibility taken for any problems caused by the use if this software.

Anyone is authorised to use this program code and methods without warranty,
all that is required is that you contact me (jeroen@happymakinggames.com),
and tell me that you use it and what you use it in -
all I want is to know is who uses it and where.

I will not ask for payment in any way: this code is free.
If you encounter a bug with the code, please tell me about it.
If you have a bug-fix for the code please provide me with it.
If you think of an improvement please tell me about it.
You are not required to provide me with bug reports, bug fixes or improvements.
If you require me to perform any work related to this code or
not related to this code, please contact me: I am always open to offers.

Pie menus are in no way my original idea, I stumbled on the idea a few years ago,
specifically Don Hopkins' http://catalog.com/hopkins/piemenus/piemenus-hopkins.html.
I am a big fan of circular menus (disc menus).
I am of the opinion that they provide the quickest method of selecting an action
from a structured tree of actions. A menu of actions organised following this method
will, in time, allow almost "blind selection" of actions. Each action selection
requires the same distance of mouse movement, just in a slightly diferent angle.
In my opinion this seems to work best with menu discs of no more than 8 items.
There is plenty of research to be found through the internet regarding the 
advantages of circular menus, for example: muscle memory.
Some websites I recommend you look at are:

http://catalog.com/hopkins/piemenus/piemenus-hopkins.html

http://www.piemenus.com/

http://www.sm.luth.se/~david/carl/www/argonaut.html

Contact: Jeroen Verbeek, jeroen@happymakinggames.com, Sydney NSW Australia
*/
/*-----------------------------------------------------------------------------------------------*/
#ifndef _CMATERIALBANK_H_
#define _CMATERIALBANK_H_


#define RGBA_BLACK		0.0f,0.0f,0.0f,0.0f
#define RGBA50_WHITE	1.0f,1.0f,1.0f,5.0f
#define RGBA_WHITE		0.0f,1.0f,1.0f,1.0f


/*-----------------------------------------------------------------------------------------------*/
/*!
Typed structure defines a single colour with floating-point R, G, B and A components.
*/
/*-----------------------------------------------------------------------------------------------*/
typedef struct GL_COLOR_RGBA
{
	float R;	/**< Red component. */
	float G;	/**< Green component. */
	float B;	/**< Blue component. */
	float A;	/**< Alpha component. */
};


/*-----------------------------------------------------------------------------------------------*/
/*!
Typed structure defines a single material.
*/
/*-----------------------------------------------------------------------------------------------*/
typedef struct GL_MATERIAL
{
	GL_COLOR_RGBA Diffuse;	/**< Contains diffuse light colour in GL_COLOR_RGBA. */
	GL_COLOR_RGBA Ambient;	/**< Contains ambient light colour in GL_COLOR_RGBA. */
	GL_COLOR_RGBA Specular;	/**< Contains specular light colour in GL_COLOR_RGBA. */
	GL_COLOR_RGBA Emissive;	/**< Contains emissive light colour in GL_COLOR_RGBA. */
	float Shininess;		/**< Contains shininess value. */
};


/*-----------------------------------------------------------------------------------------------*/
/*!
\brief	Simple OpenGL materials class.

A simple materials class with default materials for menus and methods for
managing custom materials. Uses OpenGL calls to set materials.
Use SetMaterial() and SetCustomMaterial() to set class-managed materials
with OpenGL and GLSetMaterial() to set materials when providing a material
you do not want "managed" by the materials class.
*/
/*-----------------------------------------------------------------------------------------------*/
class CMaterialBank 
{

public:
	
	enum{
		CUSTOM_MATERIAL_BLOCK_SIZE = 32,
	};

	enum MATERIALBANK_MATERIALS{
		DEFAULT_MATERIAL,	/**< Default white material. */

		/** The following materials are all used by the circular menus */
		MENU_UNAVAILABLE,	/**< Colour of segment if the segment function is unavailable. */
		MENU_NORMAL,		/**< Colour of segment when availabla but cursor is not over it. */
		MENU_OUTSIDE,		/**< Colour of segment when cursor is outside the menu disc. */
		MENU_INSIDE,		/**< Colour of segment when cursor is over it, inside the menu circle. */

		// Maximum number of materials in the standard list
		MAX_MATERIALS
	};

public:
	
	CMaterialBank(void);
	~CMaterialBank(void);

	void GLSetMaterial(const GL_MATERIAL* pMaterial);

	bool SetMaterial(int iMaterial);
	bool GetMaterial(int iMaterial, GL_MATERIAL* pMaterial);

	void AddCustomMaterial(GL_MATERIAL* pMaterial);
	bool SetCustomMaterial(int iMaterial);
	bool GetCustomMaterial(int iMaterial, GL_MATERIAL* pMaterial);

private:
		
	static const GL_MATERIAL m_DefaultMaterials[];
	std::vector<GL_MATERIAL*> m_CustomMaterials;

};


/*-----------------------------------------------------------------------------------------------*/
/*!
\brief  Global materialbank object.

This is used as the global materialbank object. Use this to manage your custom 
materials and default materials.
Include the CMaterialBank.h header in your file to enable you to use g_pMaterialBank->Function() calls.
Other instances can be created if you like, perhaps for different material collections.

\note	MUST MAKE SURE TO CREATE THE OBJECT HERE FIRST, IT IS NOT A STANDARD SINGLETON TYPE CLASS!!!
*/
/*-----------------------------------------------------------------------------------------------*/
extern CMaterialBank *g_pMaterialBank;


#endif

